import os
import glob

"""loads most of the needed parameters for the different programs (some parameters are more hiden in the files, but in principle they shouldn't be modified)"""

#directories containing the fit files
#directory_in = '../qql_cand_spectra/'
#directory_in = '../100spectra/'
#directory_in = '../SDSS_spectra0.4<z<0.5/'
directory_in = '../SDSS_spectraz<0.4/'


#directory containing the candidats after the first selection from the fit files
directory_out = './frerat_candidats/'
if not os.path.isdir(directory_out):
    os.mkdir(directory_out)


#approximative range of the spectrometer, in angstroem (see in lib.py : select_lines)
minangst = 3800
maxangst = 9200


#be careful with the next parameters, they are all "tuples list" and it is very important to distinguish their different roles !

#determines the initial template used for the crosscorrelation (see in tem.py : factory)
#tuplelist = [('H beta', 4861.33, 47.82, 10.95), ('[O III]', 4959.0, 20.19, 8.13), ('[O III]', 5007.0, 19.0, 24.77)]

tuplelist = [('H beta', 4861.33, 5.0, 10.95), ('[O III]', 4959.0, 5.0, 8.13), ('[O III]', 5007.0, 5.0, 24.77)]


#outside these borders, the spectra is hidden (see in sp.py : mask_angst)
#minborder, maxborder = 5027, 6400
#minborder, maxborder = 6720, maxangst
minborder, maxborder = 5027, maxangst


#eliminates strong lines in the sky (see in sp.py : mask_angst)
noisetuples = (5580.0, 40.0), (6300.0, 20.0)#, (7875.0, 300.0), (8400.0, 250.0), (8900.0, 250.0)


#list of the typical emission lines in a QSO (used in masks, templates, plots...)
reftuples = [('Mg II', 2798.75, 1.0, 1.0), ('O III', 3128, 1.0, 1.0), ('[Ne V]', 3346.82, 1.0, 1.0),('[Ne V]', 3426.84, 1.0, 1.0),('[O II]', 3728.48, 1.0, 1.0), ('[Fe VII]', 3759.99, 1.0, 1.0), ('[Ne III]', 3869.85, 1.0, 1.0), ('Hd', 4102.89, 1.0, 1.0), ('Hc', 4341.68, 1.0, 1.0),('[O III]', 4364.44, 1.0, 1.0), ('[He II]', 4687.0, 1.0, 1.0), ('Hb', 4862.68, 1.0, 1.0), ('[O III]', 4960.30, 1.0, 1.0), ('[O III]', 5008.24, 1.0, 1.0), ('[Fe VII]', 5160.33, 1.0, 1.0), ('[Fe VI]', 5178.05, 1.0, 1.0), ('[N I]', 5201.05, 1.0, 1.0), ('[Fe VII]', 5277.05, 1.0, 1.0), ('[Fe XIV]', 5313.05, 1.0, 1.0), ('[Cl III]', 5545.05, 1.0, 1.0), ('[Fe VII]', 5722.30, 1.0, 1.0), ('He I', 5877.29, 1.0, 1.0), ('[Fe VII]', 6087.98, 1.0, 1.0), ('[O I]', 6302.05, 1.0, 1.0), ('[O I]', 6370.05, 1.0, 1.0),('[N II]', 6549.85, 1.0, 1.0), ('Ha', 6564.61, 1.0, 1.0), ('[N II]', 6585.28, 1.0, 1.0), ('[S II]',  6718.29, 1.0, 1.0),('[S II]', 6732.67, 1.0, 1.0), ('[Ar III]', 7137.80, 1.0, 1.0), ('[O II]', 7322.05, 1.0, 1.0)]


#eliminates the strong lines in the front QSO
masktuples = [(2798.75, 20.0), (3128, 20.0), (3346.82, 20.0),(3426.84, 20.0),(3728.48, 20.0), (3759.99, 20.0), (3869.85, 20.0), (4102.89, 20.0), (4341.68, 20.0),(4364.44, 20.0), (4687.0, 20.0), (4862.68, 20.0), (4960.30, 20.0), (5008.24, 20.0), (5160.33, 20.0), (5178.05, 20.0), (5201.05, 20.0), (5277.05, 20.0), (5313.05, 20.0), (5545.05, 20.0), (5722.30, 20.0), (5877.29, 20.0), (6087.98,20.0), (6302.05, 20.0), (6370.05, 20.0), (6549.85, 20.0), (6564.61, 320.0), (6585.28, 20.0), (6718.29, 20.0), (6732.67, 20.0), (7137.80, 20.0), (7322.05, 20.0)]

#masktuples = [(5178.0, 5.0), (5201.0, 5.0), (5277.0, 5.0), (5313.0, 5.0), (5877.0, 5.0), (6302.0, 5.0), (6370.0, 5.0)]


#to choose which flux you want to use for all the further operations (see sp.py to know the different choices available)
fluxchoice = 'fluxes_csc'


#number of peaks to detect (with the actual parameters, between 80 and 110 per cross correlation) ; the number of detected peaks depends essentialy on the initial template (see also in lib.py : peak_detect and mainpeaksdet)
n = 1000 #with 1000, it should include all the possible peaks


#minimum signal to noise ratio for a peak to be detected (see in lib.py : sigtonoise and create_cand)
ratiolimit = 0.5 


#True if you want to save a png file of a "completeplot" (see in pl.py), it is not really useful
printing = False 


#generates thes lists of fits file and candidats, sorted by quality (see the programs, i.e. not the modules)
#filelist_fits = glob.glob(directory_in + '*.fit')
#filelist_cand = glob.glob(directory_out + '*.ca')
#filelist_cand = lib.sortlist(directory_out, filelist_cand)


#it counts the number of candidats created
cand_id = 0


#the number of correctly detected peaks you want to see at the moment of the plot, 0 means all the peaks
peaknumber = 0


#To choose if you want to perform a second selection on a directory of candidats or if you want to visualize the second selection, it affects seriously the results ! (see below and 2nd_selection.py)
sec_select = False


#minimum quality and number of peaks for the candidat to be written (see lib.py : create_cand)
minpeaks = 2
minqual = 6

#print len(filelist_cand) # such things should not be made here ...


#for the second selection : the directory where the first selection is written becomes the new input directory, the second selection is written in a new directory inside the previous one
if sec_select:
    directory_in = directory_out
    directory_out += '2nd_selection_4/'
    if not os.path.isdir(directory_out):
        os.mkdir(directory_out)
    if os.listdir(directory_out):
        filelist_cand = glob.glob(directory_out + '*.ca')
        print len(filelist_cand)
        filelist_cand = lib.sortlist(directory_out, filelist_cand)
    
    minpeaks = 3



