"""
Various aux functions.
There should be more bla bla at places like this ...

"""

import scipy.signal as spsig
import numpy as np
import math
import sp, li, pl, tem, ca
import copy as pythoncopy
import matplotlib.pyplot as plt
import cPickle



def peak_detect(array,n):
    """detects the n highest peaks in the array and returns their wavelength and their height in a list of two lists"""
    
    kernel = 1
    arr = spsig.medfilt(array, kernel)#permits to eventually smooth the signal, the second role of this part is to avoid the modification of the input array
    size = len(arr)
    min = np.min(arr)
 
    m = [] #it will contain the indices of the peaks
    peaksheight = []
    i = 1 #counts the number  of peaks detected
    while i <=  n:
        
        index = np.argmax(arr)
        peakheight = max(arr)
        if index + 1 < size and index - 1 >= 0 and arr[index+1] != min and arr[index-1] != min: #the two first conditions avoid errors with the two last ; 
            i += 1
            m.append(index+1)
            peaksheight.append(peakheight)
        delta1 = - 1
        delta2 = - 1
        j = 0
        while delta2 < 1 and delta1 < 1:#the choice of '1' is arbitrary, it avoids that a very small increase is detected as a peak, be conscious that this value makes sense to detect peaks in the cross correlation, for a spectra for instance it should be for example '0.05'
            j += 1 # it represents somehow the width of the peak
        
            if index - j > 0:
                delta1 = arr[index-j-1]-arr[index-j]
            else: 
                delta1 = 1000 #it is to go out of the 'while' structure
                arr[index] = min
            if index + j < size -1:
                delta2 = arr[index+j+1]-arr[index+j]
            else:
                delta2 = 1000
                arr[index] = min
         
        arr[index-j:index+j+1] = min
        if np.all(arr == min): #it means there is no more peaks to find
            i = n + 1
        
    m = np.array(m)
    peaksheight = np.array(peaksheight)
    return np.row_stack((m, peaksheight))#eventuellement donner la largeur, the peaks are automatically sorted from to highest to the smallest
    


def mean_spectra(spectralist):
    """calculates the average flux (with and without continum) of a spectra listand returns a spectra"""
    
    m = []
    specnumber = len(spectralist)
    for j in range(specnumber):
        m.append(spectralist[j].ztopix(spectralist[j].zsdss))

    flux_cs, flux, noise = [], [], []
    for i in range(len(spectralist[0].fluxes_cs)):
        a, b, c = [], [], []
        for j in range(len(spectralist)):
            index = i+m[j]
            if index < len(spectralist[j].fluxes_cs):
                a.append(spectralist[j].fluxes_cs[index])
                b.append(spectralist[j].fluxes[index])
                c.append(spectralist[j].noise[i])
        flux_cs.append(np.mean(a))
        flux.append(np.mean(b))
        noise.append(np.mean(c))

    spec = sp.spectra(np.array(flux_cs),np.array( flux), np.array(noise))
    name = 'mean spectra from ' + str(specnumber) + ' files'
    spec.name = name
    spec.setangst()
    spec.calc_csc()
    return spec


def candidat_search(directory_in, directory_out, file, tuplelist, cand_id, minpeaks, minqual, minborder, maxborder, noisetuples = [], masktuples = [], fluxchoice = 'fluxes_csc', n = 10, ratiolimit = 0.5):
    """from a fit file, it writes candidats files (if they satisfy the selection criteria). It returns the number of candidats created. This function is quite complicated, it calls many other functions !"""
    cand_number = 0
    spec = sp.spectra()
    spec = spec.read(directory_in, file)
    
    spec.initmask()
    spec.mask_borders(minborder, maxborder)
    spec.mask_angst(noisetuples)
    spec.mask_angst(masktuples, spec.zsdss)    

    template = tem.factory(tuplelist)
     
    ccsp = sp.crosscorrspec(spec, template)
    ccsp.crosscorr(fluxchoice)
        
    peaks = peak_detect(ccsp.corr, n)
    print len(peaks[0]),
    if peaks != []:
        mainpeaks = mainpeaksdet(peaks, spec)
        print len(mainpeaks[0]),
        z = mainpeaks[0]

        for i in range(len(mainpeaks[0])):
            temp = template.copy()
            temp.setglobalz(z[i])
            temp.optimize(ccsp.spectra, fluxchoice)
            
            cand_number += create_cand(spec, temp, z[i], directory_out, ratiolimit, cand_id, cand_number, minpeaks, minqual, minborder, maxborder)
    return cand_number


def create_cand(spec, temp, z, directory_out, ratiolimit, cand_id, cand_number, minpeaks, minqual, minborder, maxborder):
    """This is more precisely the function that creats the candidats files, it applies the criteria of selection"""
    ratio = sigtonoise(spec, temp)
    number = 0        
        
    lines = []
    quality = 0
        
    for j in range(len(ratio)):
        if ratio[j] > ratiolimit:
            temp.linelist[j].strength = ratio[j]
            lines.append(temp.linelist[j])
            quality += ratio[j]
            
    peaksnumber = len(lines)
    if peaksnumber >= minpeaks and quality >= minqual: #ou 0
                
        number += 1
                
        name = str(quality)[0:5] + '_' + str(peaksnumber).zfill(2) + '_' +  str(cand_id+cand_number+number).zfill(5) + '.ca'
        cand = ca.candidat(name, spec, lines, z, quality)
        cand.write(directory_out)

    return number
            
   
def sigtonoise(spectra, tem):
    """it returns a list containing the ratio between the signal and the noise for the lines in the template"""
    ratiolist = []
    
    for i in range(len(tem.linelist)):
        
        width = tem.linelist[i].width
        centerangst = tem.linelist[i].angst
        
        n =  spectra.angsttopix(centerangst)
        i1 = spectra.angsttopix(centerangst)-spectra.angsttopix(centerangst-width)
        i2 = spectra.angsttopix(centerangst+width) - spectra.angsttopix(centerangst)
        
        av_noise = np.mean(spectra.noise[n - i1:n + i2 + 1])
        if av_noise != 0:
            ratiolist.append(tem.linelist[i].height/av_noise)
        else:
            ratiolist.append(0)
    
    return ratiolist

def mainpeaksdet(peaks, sp):
    """returns a list of the 'real' peaks in a signal, it checks that two peaks very close to each other are not considered as two different ones ; the minimum distance bewteen to peaks is arbitrary"""
    z = sp.pixtoz(peaks[0])
    heights = peaks[1]
    mainpeaks = [[z[0]],[heights[0]]] 
        
    for i in range(len(z)-2):
        deltax1 = np.absolute(z[i+1] - z[i])
        if deltax1 > 0.03:  
            mainpeaks[0].append(z[i+1])
            mainpeaks[1].append(heights[i+1])
    return mainpeaks

def sortlist(directory, filelist):
    """returns a list of candidats sorted by quality"""
    quality = []
    for file in filelist:
        qual = file[len(directory):len(directory)+5] #this is not a very safe way to read the quality from the candidat
        quality.append(float(qual))
    index = np.argsort(quality)
    index = index[::-1]
    filelist = np.array(filelist)
    return filelist[index]

    

def meanfilt(arr, N): 
    """filters a signal using a mean fliter ; N must be odd [not used]"""
    n = (N-1)/2 
    size = len(arr)
    a = []
    for i in range(size):
        if i < n :
            a.append(np.mean(arr[0:i+n+1]))
        elif i > size - n - 1 :
            a.append(np.mean(arr[i-n:size]))
        else:
            a.append(np.mean(arr[i-n:i+n+1]))
    return np.array(a)
    
    
def select_lines(reftuples, z, minangst, maxangst):
    """it is to check that the wavelengths contained in the tuples are in the range of the spectrometer"""
    useful_tuples = []
    
    for tuple in reftuples:
        if tuple[1] * (1 + z) > minangst and tuple[1] * (1 + z) < maxangst:
            useful_tuples.append(tuple)

    return useful_tuples
    
