"""
Functions to make plots
"""

import matplotlib.pyplot as plt
from matplotlib.colors import ColorConverter
from matplotlib.backends.backend_agg import FigureCanvasAgg
import sys
import scipy.signal as spsig


import sp, lib, tem

import numpy as np


def mix(spectralist, fluxchoice = 'fluxes_csc' , templatelist=[], crosslist = [], peakslist = [], indtemp = True, printing = False):
    """plots a list of spectra, templates and correlations [not used]"""
    
    for i in range(len(spectralist)):
        completeplot(spectralist[i], fluxchoice, templatelist[i], crosslist[i], peakslist[i], mainpeaks[i], printing)
        
    if not printing:
        plt.show()
        
def plotlines(template):
    """plots the lines in a template as vertical lines with a name"""
    
    for line in template.linelist:
        plt.axvline(line.angst, color = template.plotcolour, dashes = template.dashes)
			#plt.text(line.angst+20.0, 0.0, line.name, fontsize=10, color=lines.plotcolour)
        colorConverter = ColorConverter()
        lightcolor = np.array(ColorConverter.to_rgb(colorConverter, template.plotcolour))[0:3]+np.array([0.5, 0.5, 0.5])
        plt.text(line.angst, 0.0, line.name, fontsize=12, color=template.plotcolour, horizontalalignment='center', verticalalignment='top')

def plottemplate(template, angst, labels = [], withtemp = True):
    """plots a template"""
    if withtemp:
        plt.plot(angst, map(template.flux_gauss,angst), label = 'template')
        plotlines(template)
        labels.append('template')
    

def plotpeaks(peaks,  labels = [], color = 'red', withpeaks = True):
    """plots the peaks as vertical lines"""
    if withpeaks:
        for peak in peaks[0]:
            plt.axvline(peak, color = color, dashes = (5,5))

def plotspectra(spectra, labels = [], fluxchoice = 'fluxes_csc', withnoise = True, withmask = True):
    """plots a spectra, the noise, the mask"""
    """
    a = spectra.allfluxes[fluxchoice].copy()
    a[spectra.mask] = 0
    a = spsig.medfilt(a, 1)
"""
    plt.plot(spectra.angst, spectra.allfluxes[fluxchoice], color=spectra.plotcolour, label = fluxchoice)
    labels.append(fluxchoice)
    if withnoise:
        plt.plot(spectra.angst, spectra.noise, color = 'blue', label = 'noise')
        labels.append('noise')
    if withmask:
        plotmask(spectra)

    plt.xlabel("Wavelength [Angstroem]")
    
    #index = [withfluxes_cs, withfluxes, withnoise, withmask]
    
    


def plotcorr(ccsp, labels = [], withcorr = True):
    """plots the cross correlation"""
    if withcorr:
        corr = ccsp.corr
        x = range(len(corr))
        z = map(ccsp.spectra.pixtoz, x)
        plt.plot(z, corr, color = 'black', label = 'cross correlation')
        plt.xlabel("redshift")
        labels.append('cross correlation')
"""
def plotcorrlist(ccsplist, labels = []):
    for item in ccsplist:
        plotcorr(item, labels)
"""

def plotcont(spec):
    """plots the flux without continuum, once for the fit file and once using the median filter. It also shows the difference between the two fluxes"""
    plt.plot(spec.angst,spec.fluxes_cs)
    plt.plot(spec.angst,spec.fluxes_csc)
    plt.plot(spec.angst, spec.fluxes_csc - spec.fluxes_cs)
    plt.legend(('cs', 'csc', 'csc-cs'), 'upper right')

def completeplot(spec, fluxchoice,  temp, ccsp, peaks, mainpeaks, candidat,  printing):
    """plots whatever can be plotted ! """
    
    fig = plt.figure(figsize = (14,11))
    
    labels = []

    plt.subplot(311)

    plotcont(spec)


    plt.subplot(312)
    plotspectra(spec, labels, fluxchoice)
    
    plottemplate(temp,spec.angst, labels)       
        
    plt.legend(labels, 'upper right' )
    
    s = 'main z : ' + str(spec.zsdss)
    xy = (0.7, 0.1)
    plt.text(4000, -2, s)
    plt.annotate(s, xy, xycoords='figure fraction')
    plt.title(spec.name)
    

    plt.subplot(313)
    labels = []
    plotcorr(ccsp, labels )
    peaks = map(ccsp.spectra.pixtoz, peaks)
    plotpeaks(peaks, labels )
    plotpeaks(mainpeaks,  labels,  'blue')
    
    plt.legend(labels, 'upper right' )

    temparam = temp.getparam()
    s = ''
    for i in range(len(temparam)):
        s += 'height ' + str(i+1) + ' : ' + str(temparam[i]) + '\n'
    s += ' secondary z : ' + str(candidat.secz)
    xy = (0.7, 0.1)
    #plt.text(0.1, 1, s)
    plt.annotate(s, xy, xycoords='figure fraction')
    if printing:
        canvas = FigureCanvasAgg(fig)
        canvas.print_png('../figs/' + spec.name + '.png')

    
def plotcand(cand, directory, reftuples, fluxchoice, minangst, maxangst, withmask = True):
    """plots what can be interesting to check a candidat"""
    
    spec = cand.spec
    z = spec.zsdss
    
    template = tem.template(cand.lines)
    fig = plt.figure(figsize = (14,9))
    
    labels = []
    ax1 = plt.subplot(211)

    plotcont(spec)
    
    useful_tuples = lib.select_lines(reftuples, z, minangst, maxangst)
    reftemplate = tem.factory(useful_tuples)
    reftemplate.setglobalz(z)
    plottemplate(reftemplate, spec.angst, labels)       
    plt.title(spec.name)

    plt.subplot(212, sharex = ax1, sharey = ax1)
    labels = []
    plotspectra(spec, labels, fluxchoice, withmask = withmask)
    plottemplate(template, spec.angst, labels) 
    
    
    
    plt.legend(labels, 'upper right' )
    
    s = 'main z : ' + str(z) + '\n'
    s += ' secondary z : ' + str(cand.secz) + '\n'
    s += 'quality : ' + str(cand.quality)

    xy = (0.7, 0.1)
    plt.annotate(s, xy, xycoords='figure fraction')
    plt.title(cand.name)
    
def plotmask(spec):
    """plots the mask as gray areas"""
    angst = spec.angst.copy()
    angst[spec.mask] = 0
    
    x1 = []
    x2 = []
    if angst[0] == 0:
        x1.append(spec.angst[0])

    for i in range(len(angst)-1):
        x = angst[i]
        if x != 0 and angst[i+1] == 0:
            x1.append(x)
        if x == 0 and angst[i+1] != 0:
            x2.append(spec.angst[i])

    if angst[len(angst)-1] == 0:
        x2.append(spec.angst[-1])

    
    for i in range(len(x1)):
        plt.axvspan(x1[i], x2[i], facecolor='grey')
