"""
Template
"""

import li
import numpy as np
import copy as pythoncopy
from scipy.optimize import leastsq

class template:
    """Groups lines from which we create a template gaussian flux"""
    def __init__(self, linelist = None, plotcolour="black", dashes=(5,5)):
        if linelist == None:
            linelist = []
        self.linelist = linelist
        #print len(linelist)
        self.plotcolour = plotcolour
        self.dashes = dashes

    def copy(self):
        """Returns a copy of the object"""
        return pythoncopy.deepcopy(self)

    def flux_gauss(self,l):
        """gives the flux at wavelength l, corresponding to the sum of all the lines"""
        retflux = 0.0
        for line in self.linelist:
            retflux += line.flux_gauss(l)
        return retflux

    def setglobalz(self, z):
        """sets the same redshift for all the lines in the template"""
        for line in self.linelist:
            line.setz(z)

    """
    #the two next functions can be used if you want to fit not only the height of the template, but also its width ; the optimize function should works functions
    def getparam(self):
        param = np.array([])
        for line in self.linelist:
            param = np.append(param, [line.height, line.width])
        return param

    def setparam(self, param):
        for i in range(len(param)/2):
            self.linelist[i].height = param[2 * i]
            self.linelist[i].width = param[2 * i + 1]
"""
    
    def getparam(self):
        """returns a list containing the height of each line in the template"""
        param = np.array([])
        for line in self.linelist:
            param = np.append(param, line.height)
        return param

    def setparam(self, param):
        """it takes a list of heights which are set in the lines"""
        for i in range(len(param)):
            self.linelist[i].height = param[i]
       

    def optimize(self, sp, fluxchoice, withmask = True):
        """optimize the template to fit with the spectra (the parameters are the heights of the lines)"""
        a = []
        for line in self.linelist:
            a.append(line.angst)
        minangst = min(a)
        maxangst = max(a)
        
        index1 = sp.angsttopix(minangst-100)
        index2 = sp.angsttopix(maxangst+100)
        size = len(sp.allfluxes[fluxchoice])
        if index1 < 0: 
            index1 = 0
            
        if index2 > size:
            index2 = size

        x = sp.angst[index1:index2]
        a = sp.allfluxes[fluxchoice].copy()
        if withmask:
            a[sp.mask] = 0
        y = a[index1:index2]
        
        if len(x) > 10:#this is to avoid certain problems at the borders
            p0 = self.getparam()
            
            def f(x, p):
                self.setparam(p)
                return self.flux_gauss(x)
            def residus(p,y,x):
                return y-f(x,p)

            result = leastsq(residus, p0, args=(y, x), maxfev=20000)
            self.setparam(result[0])

def factory(tuplelist):
    """factory function for template : from a list of tuples (with 4 items), it returns a template"""
    
    newlines = template()
    
    for item in tuplelist:
        newline = li.line(item[0], item[1], item[2], item[3])
        newlines.linelist.append(newline)
        
    return newlines
